@@ -1,4 +1,4 @@
-/* $Id: aggregate.c,v 1.10 2001/11/01 17:02:50 jabley Exp $
+/* $Id: aggregate.c,v 1.11 2001/11/02 15:49:18 jabley Exp $
  * 
  * Copyright (c) 2000-2001 by Metromedia Fiber Network Services, Inc.
  * 
@@ -172,7 +172,7 @@ int my_inet_aton(const char *cp, struct in_addr *addr)
 
 int main(int argc, char **argv)
 {
-  int verbose = 0;
+  int verbose = 0, quiet = 0, truncate = 0;
   int default_prefix_length = 0;
   int max_prefix_length = 32;
   int max_prefix_opt = 32;
@@ -187,7 +187,7 @@ int main(int argc, char **argv)
 
   /* parse command-line switches */
 
-  while ((ch = getopt(argc, argv, "vm:o:p:")) != -1)
+  while ((ch = getopt(argc, argv, "tqvm:o:p:")) != -1)
     switch (ch)
     {
       case 'm':
@@ -209,24 +209,34 @@ int main(int argc, char **argv)
             default_prefix_length);
         break;
 
+      case 'q':
+        quiet = 1;
+        break;
+
+      case 't':
+        truncate = 1;
+        break;
+
       case 'v':
         verbose = 1;
         break;
 
       case '?':
-        moanf(1, "Syntax: aggregate [-v] [-m max_length]");
+        moanf(1, "Syntax: aggregate [-m max-length] [-o max-opt-length] [-p default-length] [-q] [-t] [-v]");
         break;
     }
 
-  if (default_prefix_length > max_prefix_length)
-    moanf(0, "default prefix length (%d) is less than maximum prefix length (%d)", \
+  if (default_prefix_length > max_prefix_length && !quiet)
+    moanf(0, "default prefix length (%d) is greater than maximum prefix length (%d)", \
       default_prefix_length, max_prefix_length);
 
-  if (verbose)
+  if (!quiet)
     moanf(0, "maximum prefix length permitted will be %d", max_prefix_length);
-  if (max_prefix_opt > max_prefix_length && verbose)
+
+  if (max_prefix_opt > max_prefix_length && !quiet)
     moanf(0, "no optimisation for prefixes longer than %d bits", max_prefix_length);
-  if (default_prefix_length && verbose)
+
+  if (default_prefix_length && !quiet)
     moanf(0, "prefix length of %d bits will be used where none specified", \
       default_prefix_length);
 
@@ -247,7 +257,8 @@ int main(int argc, char **argv)
         i < MAX_buf) i++;
       if (i == MAX_buf)
       {
-        moanf(0, "[line %d] line too long; ignoring line", line);
+        if (!quiet)
+          moanf(0, "[line %d] line too long; ignoring line", line);
         continue;
       }
       buf[i] = 0;
@@ -257,7 +268,8 @@ int main(int argc, char **argv)
       while (buf[i] != '/' && i < MAX_buf) i++;
       if (i == MAX_buf)
       {
-        moanf(0, "[line %d] line too long; ignoring line", line);
+        if (!quiet)
+          moanf(0, "[line %d] line too long; ignoring line", line);
         continue;
       }
       buf[i] = 0;
@@ -265,24 +277,35 @@ int main(int argc, char **argv)
       masklen = atoi(buf + i + 1);
       if (masklen < 1 || masklen > max_prefix_length)
       {
-        moanf(0, "[line %d] mask length %d out of range; ignoring line", \
-          line, masklen);
+        if (!quiet)
+          moanf(0, "[line %d] mask length %d out of range; ignoring line", \
+            line, masklen);
         continue;
       }
     }
 
     if (my_inet_aton(buf, &prefix) == 0)
     {
-      moanf(0, "[line %d] can't parse prefix '%s'; ignoring line", line, \
-        buf);
+      if (!quiet)
+        moanf(0, "[line %d] can't parse prefix '%s'; ignoring line", line, \
+          buf);
       continue;
     }
 
     if ((ntohl(prefix.s_addr) & (0xffffffff >> masklen)) != 0 && masklen < 32)
     {
-      moanf(0, "[line %d] %s/%d inconsistent; ignoring line", \
-        line, inet_ntoa(prefix), masklen);
-      continue;
+      if (truncate) {
+        prefix.s_addr = htonl(ntohl(prefix.s_addr) & \
+          (0xffffffff << (32-masklen)));
+        if (!quiet)
+          moanf(0, "[line %d] prefix %s/%d truncated to %s/%d", \
+            line, buf, masklen, inet_ntoa(prefix), masklen);
+      } else {
+        if (!quiet)
+          moanf(0, "[line %d] %s/%d inconsistent; ignoring line", \
+            line, inet_ntoa(prefix), masklen);
+        continue;
+      }
     }
 
     /* create a list entry, and insertion sort it */
@@ -328,8 +351,11 @@ int main(int argc, char **argv)
 
   }
 
-  if (!head)
-    moanf(1, "no prefixes supplied");
+  if (!head) {
+    if (!quiet)
+      moanf(0, "no prefixes supplied");
+    exit(0);
+  }
 
   /* list of prefixes is now complete; do our crude optimisations */
 
